class Api::PostsController < ApplicationController
  include Pundit

  after_action :verify_authorized, except: :index
  after_action :verify_policy_scoped, only: :index

  respond_to :json

  def index
    @posts = policy_scope(Post)
    render json: @posts
  end

  def show
    @post = Post.find(params[:id])
    authorize @post, :show?
    render json: @post
  end

  def create
    authorize Post, :create?
    @post = current_user.posts.build(post_params)
    @post.save!
    render json: @post
  end

  def update
    post.update_attributes!(post_params)
    authorize post, :update?
    render json: post
  end

  def destroy
    post.destroy
    authorize post, :destroy?
    render json: post
  end

  private
  def post_params
    params.require(:post).permit(:title, :body, :published)
  end

  def post
    @post ||= current_user.posts.find(params[:id])
  end

end
