class PostPolicy

  attr_reader :user, :post

  def initialize(user, post)
    @user = user
    @post = post
  end

  def index?
    true
  end

  def show?
    post.published? || is_in_control?
  end

  def create?
    user.present?
  end

  def update?
    is_in_control?
  end

  def destroy?
    is_in_control?
  end

  private
  def is_in_control?
    !!( user && ( user.admin? || post.user_id == user.id ) )
  end

  class Scope

    attr_reader :user, :scope

    def initialize(user, scope)
      @user = user
      @scope = scope
    end

    def resolve
      if user.admin?
        scope.all
      else
        scope.published
      end
    end

  end

end
