require 'test_helper'

describe Api::PostsController do

  let(:user) { build_user(id: 42) }
  let(:published) { build_post(user_id: user.id) }
  let(:json) { JSON.parse(response.body) }

  before do
    User.stubs(:by_token!).returns(user)
  end

  describe '#index' do

    let(:published) { build_post(id: 42) }

    it 'displays all the published posts' do
      Post.expects(:published).returns([published])
      get :index, token: user.token

      json.length.must_equal 1

      u = json[0]
      u.must_equal_json PostSerializer.new(published)
    end

    context 'as an admin' do

      before do
        user.admin = true
        Post.expects(:all).returns([published])
      end

      it 'displays all posts' do
        get :index, token: user.token
      end

    end

  end

  describe '#show' do

    let(:published) { build_post(id: 42) }

    before do
      Post.expects(:find).with('42').returns(published)
    end

    it 'shows a published' do
      get :show, id: 42, token: user.token

      json.must_equal_json PostSerializer.new(published)
    end

  end

  describe '#create' do

    it 'creates a new published' do
      Post.count.must_equal 0

      post :create, post: {
        title: 'New published Title', body: 'some published body'
      }

      Post.count.must_equal 1
      json.must_equal_json PostSerializer.new(Post.first)
    end

  end

  describe '#update' do

    before do
      published.save!
    end

    it 'updates a post' do
      put :update, id: published.id, post: {title: 'Mark Bates'}

      published.reload
      json.must_equal_json PostSerializer.new(published)
      published.title.must_equal 'Mark Bates'
    end

  end

  describe '#destroy' do

    before do
      published.save!
    end

    it 'destroys a published' do
      Post.count.must_equal 1

      delete :destroy, id: published.id

      Post.count.must_equal 0
      json.must_equal_json PostSerializer.new(published)
    end

  end

end
