require 'test_helper'

describe PostPolicy do

  let(:post) { build_post(user_id: 42, published: true) }
  let(:policy) { PostPolicy.new(user, post) }

  context 'with post owner' do

    let(:user) { build_user(id: 42, admin: false) }

    it 'can index' do
      policy.index?.must_equal true
    end

    it 'can create' do
      policy.create?.must_equal true
    end

    it 'can destroy' do
      policy.destroy?.must_equal true
    end

    it 'can update' do
      policy.update?.must_equal true
    end

    it 'can see a published post' do
      policy.show?.must_equal true
    end

    it 'can see an unpublished post' do
      post.published = false
      policy.show?.must_equal true
    end

  end

  context 'without post owner' do

    let(:user) { build_user(id: 69, admin: false) }

    it 'can index' do
      policy.index?.must_equal true
    end

    it 'can create' do
      policy.create?.must_equal true
    end

    it 'cant destroy' do
      policy.destroy?.must_equal false
    end

    it 'cant update' do
      policy.update?.must_equal false
    end

    it 'can see a published post' do
      policy.show?.must_equal true
    end

    it 'cant see an unpublished post' do
      post.published = false
      policy.show?.must_equal false
    end

  end

  context 'with admin user' do

    let(:user) { build_user(id: 69, admin: true) }

    it 'can index' do
      policy.index?.must_equal true
    end

    it 'can create' do
      policy.create?.must_equal true
    end

    it 'can destroy' do
      policy.destroy?.must_equal true
    end

    it 'can update' do
      policy.update?.must_equal true
    end

    it 'can see a published post' do
      policy.show?.must_equal true
    end

    it 'can see an unpublished post' do
      post.published = false
      policy.show?.must_equal true
    end

  end

  context 'with no user' do

    let(:user) { nil }

    it 'can index' do
      policy.index?.must_equal true
    end

    it 'cant create' do
      policy.create?.must_equal false
    end

    it 'cant destroy' do
      policy.destroy?.must_equal false
    end

    it 'cant update' do
      policy.update?.must_equal false
    end

    it 'can see a published post' do
      policy.show?.must_equal true
    end

    it 'cant see an unpublished post' do
      post.published = false
      policy.show?.must_equal false
    end

  end

  describe PostPolicy::Scope do

    let(:scope) { PostPolicy::Scope.new(user, Post) }

    context 'with admin user' do

      let(:user) { build_user(id: 69, admin: true) }

      it 'returns the top level scope' do
        scope.resolve.must_equal Post.all
      end

    end

    context 'without admin user' do

      let(:user) { build_user(id: 69, admin: false) }

      it 'returns published scope' do
        scope.resolve.must_equal Post.published
      end

    end

  end

end
