require 'test_helper'

describe UserSerializer do

  let(:user) { build_user(id: 42) }
  let(:scope) { build_user(admin: false) }
  let(:serializer) { UserSerializer.new(user, scope: scope) }
  let(:json) { serializer.as_json }

  [:id, :name, :email, :admin].each do |attr|

    describe attr do

      it 'is present' do
        json[attr].must_equal user.send(attr)
      end

    end

  end

  describe '#_links' do

    it 'generates _links for the user api' do
      serializer._links.must_equal({
        collection: 'http://test.host/api/users',
        self: 'http://test.host/api/users/42'
      })
    end

    it 'is included in the json' do
      json[:_links].must_equal serializer._links
    end

  end

  describe '#token' do

    context 'as an admin' do

      let(:scope) { build_user(admin: true) }

      it 'includes the token' do
        json[:token].must_equal user.token
      end

    end

    context 'not an admin' do

      it 'does not include the token' do
        json.key?(:token).must_equal false
      end

    end

  end

end
