require 'test_helper'

describe Api::UsersController do

  let(:user) { build_user }
  let(:json) { JSON.parse(response.body) }

  before do
    User.stubs(:by_token!).returns(user)
  end

  describe '#index' do

    let(:user) { build_user(id: 42) }

    before do
      User.expects(:all).returns([user])
    end

    it 'displays all the users' do
      get :index, token: user.token

      json.length.must_equal 1

      u = json[0]
      u.must_equal_json UserSerializer.new(user)
    end

  end

  describe '#show' do

    let(:user) { build_user(id: 42) }

    before do
      User.expects(:find).with('42').returns(user)
    end

    it 'shows a user' do
      get :show, id: 42, token: user.token

      json.must_equal_json UserSerializer.new(user)
    end

  end

  describe '#create' do

    it 'creates a new user' do
      User.count.must_equal 0

      post :create, user: {
        name: 'Mark Bates', email: 'mark@example.com'
      }

      User.count.must_equal 1
      json.must_equal_json UserSerializer.new(User.first)
    end

  end

  describe '#update' do

    before do
      user.save!
    end

    it 'updates a user' do
      put :update, id: user.id, user: {name: 'Mark Bates'}

      user.reload
      json.must_equal_json UserSerializer.new(user)
      user.name.must_equal 'Mark Bates'
    end

  end

  describe '#destroy' do

    before do
      user.save!
    end

    it 'destroys a user' do
      User.count.must_equal 1

      delete :destroy, id: user.id

      User.count.must_equal 0
      json.must_equal_json UserSerializer.new(user)
    end

  end

end
